%%====================================================================
%%
%% Leo Object Storage
%%
%% Copyright (c) 2012-2017 Rakuten, Inc.
%%
%% This file is provided to you under the Apache License,
%% Version 2.0 (the "License"); you may not use this file
%% except in compliance with the License.  You may obtain
%% a copy of the License at
%%
%%   http://www.apache.org/licenses/LICENSE-2.0
%%
%% Unless required by applicable law or agreed to in writing,
%% software distributed under the License is distributed on an
%% "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
%% KIND, either express or implied.  See the License for the
%% specific language governing permissions and limitations
%% under the License.
%%
%%====================================================================
-module(leo_object_transformer_tests).
-author('yosuke hara').

-include_lib("eunit/include/eunit.hrl").
-include("leo_object_storage.hrl").


-ifdef(EUNIT).

leo_object_transformer_testssuit_test_() ->
    {setup,
     fun ( ) ->
             ok
     end,
     fun (_) ->
             ok
     end,
     [
      {"Tests all transformers",
       {timeout, 1000, fun transform/0}}
     ]}.


transform() ->
    Key = <<"leofs/distributed/storage">>,
    AddrId = 328119305811,
    KSize = erlang:byte_size(Key),
    DSize = 128,
    MSize = 384,
    CMeta = <<"user-defined-metadata">>,
    CSize = 64,
    CNum = 1,
    CIndex = 1,
    Offset = 512,
    Clock = leo_date:clock(),
    Timestamp = leo_date:now(),
    Checksum = 1549499612212564499221795376818021,
    RingHash = 1116532140187397365077604119361862,
    ClusterId = 'leofs_cluster_1',
    NumOfReplicas = 3,
    Preferred_R = 1,
    Preferred_W = 2,
    Preferred_D = 2,
    Ver = 1,
    Del = 0,

    #?OBJECT{key = Key,
             addr_id = AddrId,
             ksize = KSize,
             dsize = DSize,
             msize = MSize,
             csize = CSize,
             cnumber = CNum,
             cindex = CIndex,
             offset = Offset,
             clock = Clock,
             timestamp = Timestamp,
             checksum = Checksum,
             ring_hash = RingHash,
             del = Del} = leo_object_storage_transformer:metadata_to_object(#metadata{key = Key,
                                                                                      addr_id = AddrId,
                                                                                      ksize = KSize,
                                                                                      dsize = DSize,
                                                                                      msize = MSize,
                                                                                      csize = CSize,
                                                                                      cnumber = CNum,
                                                                                      cindex = CIndex,
                                                                                      offset = Offset,
                                                                                      clock = Clock,
                                                                                      timestamp = Timestamp,
                                                                                      checksum = Checksum,
                                                                                      ring_hash = RingHash,
                                                                                      del = Del}),
    #?OBJECT{key = Key,
             addr_id = AddrId,
             ksize = KSize,
             dsize = DSize,
             msize = MSize,
             csize = CSize,
             cnumber = CNum,
             cindex = CIndex,
             offset = Offset,
             clock = Clock,
             timestamp = Timestamp,
             checksum = Checksum,
             ring_hash = RingHash,
             cluster_id = ClusterId,
             num_of_replicas = NumOfReplicas,
             ver = Ver,
             del = Del} = leo_object_storage_transformer:metadata_to_object(#metadata_1{key = Key,
                                                                                        addr_id = AddrId,
                                                                                        ksize = KSize,
                                                                                        dsize = DSize,
                                                                                        msize = MSize,
                                                                                        csize = CSize,
                                                                                        cnumber = CNum,
                                                                                        cindex = CIndex,
                                                                                        offset = Offset,
                                                                                        clock = Clock,
                                                                                        timestamp = Timestamp,
                                                                                        checksum = Checksum,
                                                                                        ring_hash = RingHash,
                                                                                        cluster_id = ClusterId,
                                                                                        num_of_replicas = NumOfReplicas,
                                                                                        ver = Ver,
                                                                                        del = Del}),
    #?OBJECT{key = Key,
             addr_id = AddrId,
             ksize = KSize,
             dsize = DSize,
             meta = CMeta,
             msize = MSize,
             csize = CSize,
             cnumber = CNum,
             cindex = CIndex,
             offset = Offset,
             clock = Clock,
             timestamp = Timestamp,
             checksum = Checksum,
             ring_hash = RingHash,
             cluster_id = ClusterId,
             num_of_replicas = NumOfReplicas,
             ver = Ver,
             del = Del} = leo_object_storage_transformer:metadata_to_object(#metadata_2{key = Key,
                                                                                        addr_id = AddrId,
                                                                                        ksize = KSize,
                                                                                        dsize = DSize,
                                                                                        meta = CMeta,
                                                                                        msize = MSize,
                                                                                        csize = CSize,
                                                                                        cnumber = CNum,
                                                                                        cindex = CIndex,
                                                                                        offset = Offset,
                                                                                        clock = Clock,
                                                                                        timestamp = Timestamp,
                                                                                        checksum = Checksum,
                                                                                        ring_hash = RingHash,
                                                                                        cluster_id = ClusterId,
                                                                                        num_of_replicas = NumOfReplicas,
                                                                                        ver = Ver,
                                                                                        del = Del}),
    #?OBJECT{key = Key,
             addr_id = AddrId,
             ksize = KSize,
             dsize = DSize,
             meta = CMeta,
             msize = MSize,
             csize = CSize,
             cnumber = CNum,
             cindex = CIndex,
             offset = Offset,
             clock = Clock,
             timestamp = Timestamp,
             checksum = Checksum,
             ring_hash = RingHash,
             cluster_id = ClusterId,
             num_of_replicas = NumOfReplicas,
             preferred_r = Preferred_R,
             preferred_w = Preferred_W,
             preferred_d = Preferred_D,
             ver = Ver,
             del = Del} = leo_object_storage_transformer:metadata_to_object(#?METADATA{key = Key,
                                                                                       addr_id = AddrId,
                                                                                       ksize = KSize,
                                                                                       dsize = DSize,
                                                                                       meta = CMeta,
                                                                                       msize = MSize,
                                                                                       csize = CSize,
                                                                                       cnumber = CNum,
                                                                                       cindex = CIndex,
                                                                                       offset = Offset,
                                                                                       clock = Clock,
                                                                                       timestamp = Timestamp,
                                                                                       checksum = Checksum,
                                                                                       ring_hash = RingHash,
                                                                                       cluster_id = ClusterId,
                                                                                       num_of_replicas = NumOfReplicas,
                                                                                       preferred_r = Preferred_R,
                                                                                       preferred_w = Preferred_W,
                                                                                       preferred_d = Preferred_D,
                                                                                       ver = Ver,
                                                                                       del = Del}),
    ?debugVal("## Passed: metadata_to_object_test"),


    #?METADATA{key = Key,
               addr_id = AddrId,
               ksize = KSize,
               dsize = DSize,
               msize = MSize,
               csize = CSize,
               cnumber = CNum,
               cindex = CIndex,
               offset = Offset,
               clock = Clock,
               timestamp = Timestamp,
               checksum = Checksum,
               ring_hash = RingHash,
               del = Del} = leo_object_storage_transformer:object_to_metadata(#object{key = Key,
                                                                                      addr_id = AddrId,
                                                                                      ksize = KSize,
                                                                                      dsize = DSize,
                                                                                      msize = MSize,
                                                                                      csize = CSize,
                                                                                      cnumber = CNum,
                                                                                      cindex = CIndex,
                                                                                      offset = Offset,
                                                                                      clock = Clock,
                                                                                      timestamp = Timestamp,
                                                                                      checksum = Checksum,
                                                                                      ring_hash = RingHash,
                                                                                      del = Del}),
    #?METADATA{key = Key,
               addr_id = AddrId,
               ksize = KSize,
               dsize = DSize,
               meta = CMeta,
               msize = MSize,
               csize = CSize,
               cnumber = CNum,
               cindex = CIndex,
               offset = Offset,
               clock = Clock,
               timestamp = Timestamp,
               checksum = Checksum,
               ring_hash = RingHash,
               cluster_id = ClusterId,
               num_of_replicas = NumOfReplicas,
               ver = Ver,
               del = Del} = leo_object_storage_transformer:object_to_metadata(#object_1{key = Key,
                                                                                        addr_id = AddrId,
                                                                                        ksize = KSize,
                                                                                        dsize = DSize,
                                                                                        meta = CMeta,
                                                                                        msize = MSize,
                                                                                        csize = CSize,
                                                                                        cnumber = CNum,
                                                                                        cindex = CIndex,
                                                                                        offset = Offset,
                                                                                        clock = Clock,
                                                                                        timestamp = Timestamp,
                                                                                        checksum = Checksum,
                                                                                        ring_hash = RingHash,
                                                                                        cluster_id = ClusterId,
                                                                                        num_of_replicas = NumOfReplicas,
                                                                                        ver = Ver,
                                                                                        del = Del}),
    #?METADATA{key = Key,
               addr_id = AddrId,
               ksize = KSize,
               dsize = DSize,
               meta = CMeta,
               msize = MSize,
               csize = CSize,
               cnumber = CNum,
               cindex = CIndex,
               offset = Offset,
               clock = Clock,
               timestamp = Timestamp,
               checksum = Checksum,
               ring_hash = RingHash,
               cluster_id = ClusterId,
               num_of_replicas = NumOfReplicas,
               preferred_r = Preferred_R,
               preferred_w = Preferred_W,
               preferred_d = Preferred_D,
               ver = Ver,
               del = Del} = leo_object_storage_transformer:object_to_metadata(#?OBJECT{key = Key,
                                                                                       addr_id = AddrId,
                                                                                       ksize = KSize,
                                                                                       dsize = DSize,
                                                                                       meta = CMeta,
                                                                                       msize = MSize,
                                                                                       csize = CSize,
                                                                                       cnumber = CNum,
                                                                                       cindex = CIndex,
                                                                                       offset = Offset,
                                                                                       clock = Clock,
                                                                                       timestamp = Timestamp,
                                                                                       checksum = Checksum,
                                                                                       ring_hash = RingHash,
                                                                                       cluster_id = ClusterId,
                                                                                       num_of_replicas = NumOfReplicas,
                                                                                       preferred_r = Preferred_R,
                                                                                       preferred_w = Preferred_W,
                                                                                       preferred_d = Preferred_D,
                                                                                       ver = Ver,
                                                                                       del = Del}),
    ?debugVal("## Passed: object_to_metadata_test"),


    #?METADATA{key = Key,
               addr_id = AddrId,
               ksize = KSize,
               dsize = DSize,
               msize = MSize,
               csize = CSize,
               cnumber = CNum,
               cindex = CIndex,
               offset = Offset,
               clock = Clock,
               timestamp = Timestamp,
               checksum = Checksum,
               ring_hash = RingHash,
               del = Del} = leo_object_storage_transformer:transform_metadata(#metadata{key = Key,
                                                                                        addr_id = AddrId,
                                                                                        ksize = KSize,
                                                                                        dsize = DSize,
                                                                                        msize = MSize,
                                                                                        csize = CSize,
                                                                                        cnumber = CNum,
                                                                                        cindex = CIndex,
                                                                                        offset = Offset,
                                                                                        clock = Clock,
                                                                                        timestamp = Timestamp,
                                                                                        checksum = Checksum,
                                                                                        ring_hash = RingHash,
                                                                                        del = Del}),
    #?METADATA{key = Key,
               addr_id = AddrId,
               ksize = KSize,
               dsize = DSize,
               msize = MSize,
               csize = CSize,
               cnumber = CNum,
               cindex = CIndex,
               offset = Offset,
               clock = Clock,
               timestamp = Timestamp,
               checksum = Checksum,
               ring_hash = RingHash,
               cluster_id = ClusterId,
               num_of_replicas = NumOfReplicas,
               ver = Ver,
               del = Del} = leo_object_storage_transformer:transform_metadata(#metadata_1{key = Key,
                                                                                          addr_id = AddrId,
                                                                                          ksize = KSize,
                                                                                          dsize = DSize,
                                                                                          msize = MSize,
                                                                                          csize = CSize,
                                                                                          cnumber = CNum,
                                                                                          cindex = CIndex,
                                                                                          offset = Offset,
                                                                                          clock = Clock,
                                                                                          timestamp = Timestamp,
                                                                                          checksum = Checksum,
                                                                                          ring_hash = RingHash,
                                                                                          cluster_id = ClusterId,
                                                                                          num_of_replicas = NumOfReplicas,
                                                                                          ver = Ver,
                                                                                          del = Del}),
    #?METADATA{key = Key,
               addr_id = AddrId,
               ksize = KSize,
               dsize = DSize,
               meta = CMeta,
               msize = MSize,
               csize = CSize,
               cnumber = CNum,
               cindex = CIndex,
               offset = Offset,
               clock = Clock,
               timestamp = Timestamp,
               checksum = Checksum,
               ring_hash = RingHash,
               cluster_id = ClusterId,
               num_of_replicas = NumOfReplicas,
               ver = Ver,
               del = Del} = leo_object_storage_transformer:transform_metadata(#metadata_2{key = Key,
                                                                                          addr_id = AddrId,
                                                                                          ksize = KSize,
                                                                                          dsize = DSize,
                                                                                          meta = CMeta,
                                                                                          msize = MSize,
                                                                                          csize = CSize,
                                                                                          cnumber = CNum,
                                                                                          cindex = CIndex,
                                                                                          offset = Offset,
                                                                                          clock = Clock,
                                                                                          timestamp = Timestamp,
                                                                                          checksum = Checksum,
                                                                                          ring_hash = RingHash,
                                                                                          cluster_id = ClusterId,
                                                                                          num_of_replicas = NumOfReplicas,
                                                                                          ver = Ver,
                                                                                          del = Del}),
    ?debugVal("## Passed: transform_metadata_test"),


    Method = 'put',
    Data = <<"object...">>,
    ReqId = 2017042418150000,
    #?OBJECT{method = Method,
             key = Key,
             addr_id = AddrId,
             data = Data,
             meta = CMeta,
             ksize = KSize,
             dsize = DSize,
             msize = MSize,
             csize = CSize,
             cnumber = CNum,
             cindex = CIndex,
             offset =  Offset,
             clock = Clock,
             timestamp = Timestamp,
             checksum = Checksum,
             ring_hash = RingHash,
             req_id = ReqId,
             del = Del} = leo_object_storage_transformer:transform_object(#object{method = Method,
                                                                                  key = Key,
                                                                                  addr_id = AddrId,
                                                                                  data = Data,
                                                                                  meta = CMeta,
                                                                                  ksize = KSize,
                                                                                  dsize = DSize,
                                                                                  msize = MSize,
                                                                                  csize = CSize,
                                                                                  cnumber = CNum,
                                                                                  cindex = CIndex,
                                                                                  offset =  Offset,
                                                                                  clock = Clock,
                                                                                  timestamp = Timestamp,
                                                                                  checksum = Checksum,
                                                                                  ring_hash = RingHash,
                                                                                  req_id = ReqId,
                                                                                  del = Del}),
    #?OBJECT{method = Method,
             key = Key,
             addr_id = AddrId,
             data = Data,
             meta = CMeta,
             ksize = KSize,
             dsize = DSize,
             msize = MSize,
             csize = CSize,
             cnumber = CNum,
             cindex = CIndex,
             offset = Offset,
             clock = Clock,
             timestamp = Timestamp,
             checksum = Checksum,
             ring_hash = RingHash,
             req_id= ReqId,
             cluster_id = ClusterId,
             num_of_replicas = NumOfReplicas,
             ver = Ver,
             del = Del} = leo_object_storage_transformer:transform_object(#object_1{method = Method,
                                                                                    key = Key,
                                                                                    addr_id = AddrId,
                                                                                    data = Data,
                                                                                    meta = CMeta,
                                                                                    ksize = KSize,
                                                                                    dsize = DSize,
                                                                                    msize = MSize,
                                                                                    csize = CSize,
                                                                                    cnumber = CNum,
                                                                                    cindex = CIndex,
                                                                                    offset = Offset,
                                                                                    clock = Clock,
                                                                                    timestamp = Timestamp,
                                                                                    checksum = Checksum,
                                                                                    ring_hash = RingHash,
                                                                                    req_id= ReqId,
                                                                                    cluster_id = ClusterId,
                                                                                    num_of_replicas = NumOfReplicas,
                                                                                    ver = Ver,
                                                                                    del = Del}),
    ?debugVal("## Passed: transform_object_test"),


    CustomMetadataBin = leo_object_storage_transformer:list_to_cmeta_bin(
                          [{?PROP_CMETA_CLUSTER_ID, ClusterId},
                           {?PROP_CMETA_NUM_OF_REPLICAS, NumOfReplicas},
                           {?PROP_CMETA_PREFERRED_R, Preferred_R},
                           {?PROP_CMETA_PREFERRED_W, Preferred_W},
                           {?PROP_CMETA_PREFERRED_D, Preferred_D},
                           {?PROP_CMETA_VER, Ver},
                           {?PROP_CMETA_UDM, CMeta}
                          ]),
    {ok, {Metadata_1, CMeta_1}} = leo_object_storage_transformer:cmeta_bin_into_metadata(
                                    CustomMetadataBin,
                                    #?METADATA{key = Key,
                                               addr_id = AddrId,
                                               ksize = KSize,
                                               dsize = DSize,
                                               meta = CMeta,
                                               msize = MSize,
                                               csize = CSize,
                                               cnumber = CNum,
                                               cindex = CIndex,
                                               offset = Offset,
                                               clock = Clock,
                                               timestamp = Timestamp,
                                               checksum = Checksum,
                                               ring_hash = RingHash,
                                               del = Del}),
    ?assertEqual(ClusterId, Metadata_1#?METADATA.cluster_id),
    ?assertEqual(NumOfReplicas, Metadata_1#?METADATA.num_of_replicas),
    ?assertEqual(Preferred_R, Metadata_1#?METADATA.preferred_r),
    ?assertEqual(Preferred_W, Metadata_1#?METADATA.preferred_w),
    ?assertEqual(Preferred_D, Metadata_1#?METADATA.preferred_d),
    ?assertEqual(Ver, Metadata_1#?METADATA.ver),
    ?assertEqual(CMeta, CMeta_1),
    %% ?debugVal(Metadata_1#?METADATA.cluster_id),
    %% ?debugVal(Metadata_1#?METADATA.num_of_replicas),
    %% ?debugVal(Metadata_1#?METADATA.preferred_r),
    %% ?debugVal(Metadata_1#?METADATA.preferred_w),
    %% ?debugVal(Metadata_1#?METADATA.preferred_d),
    %% ?debugVal(Metadata_1#?METADATA.ver),
    %% ?debugVal(CMeta_1),
    ?debugVal("## Passed: cmeta_bin_into_metadata_test"),
    ok.

-endif.
